//////////////////////////
// ******************** //
//        Figures       //
// ******************** //
//////////////////////////

@mixin crop-original {
  > div {
    height: auto;
    padding-bottom: 0;
    > img {
      height: auto;
      position: relative;
    }
  }
}

@mixin crop-1x1 {
  > div {
    padding-bottom: 100%;
  }
}

@mixin crop-16x9 {
  > div {
    padding-bottom: 56.25%;
  }
}

@mixin crop-4x3 {
  > div {
    padding-bottom: 75%;
  }
}

@mixin crop-3x4 {
  > div {
    padding-bottom: 133.3333%;
  }
}

@mixin crop-3x1 {
  > div {
    padding-bottom: 33.3333%;
  }
}

.image {
  overflow: hidden;
  width: auto;
  max-width: 100%;
  position: relative;
  padding-top: 0px;
  float: none;
  clear: both;
  > div {
    position: relative;
    height: auto;
    overflow: hidden;
    > img {
      display: block;
      width: 100%;
      position: absolute;
      top: 0;
      left: 0;
      height: 100%;
    }
  }
}